@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(Net::Async::Redis::Commands IO::Async::Notifier);
 
-our $VERSION = '1.001';
+our $VERSION = '1.002';
 
 =head1 NAME
 
@@ -55,6 +55,8 @@ B<NOTE>: For a full list of Redis methods, please see L<Net::Async::Redis::Comma
 =head1 METHODS - Subscriptions
 
 See L<https://redis.io/topics/pubsub> for more details on this topic.
+There's also more details on the internal implementation in Redis here:
+L<https://making.pusher.com/redis-pubsub-under-the-hood/>.
 
 =cut
 
@@ -69,8 +71,13 @@ sub psubscribe {
 	return $self->execute_command(
 		PSUBSCRIBE => $pattern
 	)->then(sub {
-            $self->{pubsub} = 1;
-		Future->done
+        $self->{pubsub} = 1;
+        Future->done(
+            $self->{subscription_pattern_channel}{$pattern} //= Net::Async::Redis::Subscription->new(
+                redis => $self,
+                channel => $pattern
+            )
+        );
 	})
 }
 
@@ -101,7 +108,6 @@ sub subscribe {
 	my ($self, @channels) = @_;
     $self->next::method(@channels)
         ->then(sub {
-                warn "self = $self";
             $self->{pubsub} = 1;
             my @subs = map {
                 $self->{subscription_channel}{$_} //= Net::Async::Redis::Subscription->new(
@@ -256,8 +262,26 @@ sub on_message {
                     $log->errorf('Have message for unknown pattern channel [%s]', $channel);
                 }
             }
+        } elsif($type eq 'unsubscribe') {
+            if(my $sub = delete $self->{subscription_channel}{$channel}) {
+                $log->tracef('Removed subscription for [%s]', $channel);
+                delete $self->{pubsub} unless keys(%{$self->{subscription_channel}}) or keys(%{$self->{subscription_pattern_channel}});
+            } else {
+                $log->errorf('Have unsubscription for unknown channel [%s]', $channel);
+            }
+        } elsif($type eq 'punsubscribe') {
+            if(my $sub = delete $self->{subscription_pattern_channel}{$channel}) {
+                $log->tracef('Removed pattern subscription for [%s]', $channel);
+                delete $self->{pubsub} unless keys(%{$self->{subscription_channel}}) or keys(%{$self->{subscription_pattern_channel}});
+            } else {
+                $log->errorf('Have pattern unsubscription for unknown channel [%s]', $channel);
+            }
+        } elsif($type eq 'subscribe') {
+            $log->errorf('Have subscription for unknown channel [%s]', $channel) unless exists $self->{subscription_channel}{$channel};
+        } elsif($type eq 'punsubscribe') {
+            $log->errorf('Have subscription for unknown pattern channel [%s]', $channel) unless exists $self->{subscription_pattern_channel}{$channel};
         } else {
-            $log->infof('have %s with channel %s payload %s', $type, $channel, $payload);
+            $log->errorf('have unknown pubsub message type %s with channel %s payload %s', $type, $channel, $payload);
         }
 		$self->bus->invoke_event(message => $data) if exists $self->{bus};
 	} else {
